﻿using System.Runtime.CompilerServices;
using System.Runtime.Serialization;
using System.Text.Json;
using System.Text.Json.Nodes;
using System.Text.Json.Serialization;

namespace HDLNoCGen
{
    static class Program
    {
        static void Main(string[] args)
        {
            string location = "";
            string name = "";
            string action = "o";
            string new_name = "";

            try
            {
                for (int i = 0; i < args.Length; i++)
                {
                    switch (args[i])
                    {
                        case "-l":
                        case "--location":
                            location = args[++i];
                            break;
                        case "-n":
                        case "--name":
                            name = args[++i];
                            break;
                        case "-o":
                        case "--open":
                            action = "o";
                            break;
                        case "-c":
                        case "--create":
                            action = "c";
                            break;
                        case "-e":
                        case "--erase":
                            action = "e";
                            break;
                        case "-r":
                        case "--rename":
                            action = "r";
                            new_name = args[++i];
                            break;
                        default:
                            Console.WriteLine($"Argument {args[i]} is invalid");
                            Environment.Exit(1);
                            break;
                    }
                }
            }
            catch (IndexOutOfRangeException e)
            {
                Console.WriteLine("No argument after an option that requires an argument");
                Environment.Exit(1);
            }

            string metadata_location = $"{location}/{name}_metadata.json";
            string new_metadata_location = $"{location}/{new_name}_metadata.json";

            switch (action)
            {
                case "o":
                case "r":

                    if (!Directory.Exists(location))
                    {
                        Console.WriteLine("Failed to find project directory");
                        Environment.Exit(1);
                    }

                    if (!File.Exists(metadata_location))
                    {
                        Console.WriteLine("Failed to find project metadata");
                        Environment.Exit(1);
                    }

                    StreamReader sr = new StreamReader(metadata_location);
                    string json = sr.ReadToEnd();
                    sr.Close();
                    ProjectSettings projectSettings = new ProjectSettings();

                    try
                    {
                        projectSettings = JsonSerializer.Deserialize<ProjectSettings>(json);
                        if (projectSettings.projectMetadata.name != name)
                        {
                            Console.WriteLine("Wrong project name in the metadata");
                            Environment.Exit(1);
                        }
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine("Failed to parse project metadata");
                        Environment.Exit(1);
                    }

                    if (action == "r")
                    {
                        projectSettings.projectMetadata.name = new_name;
                        json = JsonSerializer.Serialize<ProjectSettings>(projectSettings, new JsonSerializerOptions { WriteIndented = true });

                        try
                        {
                            File.Delete(metadata_location);
                            StreamWriter sw = new StreamWriter(new_metadata_location);
                            sw.Write(json);
                            sw.Close();
                        }
                        catch (Exception e)
                        {
                            Console.WriteLine("Failed to rename the project");
                            Environment.Exit(1);
                        }
                    }

                    break;
                case "c":

                    if (!Directory.Exists(location))
                    {
                        try
                        {
                            Directory.CreateDirectory(location);
                        }
                        catch (Exception e)
                        {
                            Console.WriteLine("Failed to create a project directory");
                        }
                    }

                    if (Directory.EnumerateFileSystemEntries(location).Any())
                    {
                        Console.WriteLine("Failed to create a project. Directory must be empty");
                        Environment.Exit(1);
                    }

                    ProjectSettings project_settings = new ProjectSettings();
                    project_settings.projectMetadata.name = name;
                    string serialized = JsonSerializer.Serialize(project_settings, new JsonSerializerOptions { WriteIndented = true });

                    try
                    {
                        StreamWriter sw = new StreamWriter(metadata_location);
                        sw.Write(serialized);
                        sw.Close();
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine("Failed to create the project metadata");
                        Environment.Exit(1);
                    }

                    break;
                case "e":

                    if (!Directory.Exists(location))
                    {
                        Console.WriteLine("Non-existent directory");
                        Environment.Exit(0);
                    }

                    if (!File.Exists(metadata_location))
                    {
                        Console.WriteLine("Non-existent project");
                        Environment.Exit(0);
                    }

                    try
                    {
                        Directory.Delete(location, true);
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine("Failed to delete the project");
                        Environment.Exit(1);
                    }

                    break;
            }
        }
    }
}